/*
  fornext.prg
  test program
  compile : hbmk2 fornext -run
*/
PROCEDURE Main()
   LOCAL lResult

   CLS
   ? "`1...9` Step `3`"
   ?
   lResult := hb_ForNext2(1, 9, 3, {|n| QQout(n) })
   ? " * lResult must be: .T. is: ", lResult
   ?

   ? "`9...1` Step `-3`"
   ?
   lResult := hb_ForNext2(9, 1, -3, {|n| QQout(n) })
   ? "* lResult must be: .T. is: ", lResult
   ?
   ? "`1...1` Step `-1`"
   ?
   lResult := hb_ForNext2(1, 1, -1, {|n| QQout(n) })
   ? " * lResult must be: .T. is: ", lResult
   ?
   ? "`9...1` Step `0`"
   ?
   lResult := hb_ForNext2(9, 1, 0, {|n| QQout(n) })
   ? " * lResult must be: `.F.` is ", lResult
   ?
   ? "`1...3` Step `-1`"
   ?
   lResult := hb_ForNext2(1, 3, -1, {|n| QQout(n) })
   ? " * lResult must be: `.F.` is ", lResult
   ?
   ? "`-1...-3` Step `-1`"
   ?
   lResult := hb_ForNext2(-1, -3, -1, {|n| QQout(n) })
   ? " * lResult must be: `.T.` is ", lResult
   ?
   ? "`<NIL>...5` Step `<NIL>`"
   ?
   lResult := hb_ForNext2( , 5, , {|n| QQout(n) } )
   ? " * lResult must be: `.T.` is ", lResult
   ?
   RETURN


#pragma begindump

/*
 * HB_FORNEXT2()
 * a modified version of HB_FORNEXT()
 * plain emulation of for(...) C loop
 * Jan 2020 - Pete D. - <pete_westg@yahoo.com>
 */

#include "hbapi.h"
#include "hbvm.h"

/* hb_ForNext2( [<nStart>], nEnd, [<nStep>], bCodeBlock ) --> lResult
     - <nStart> default: 0
     - <nStep>  default: 1
*/
HB_FUNC( HB_FORNEXT2 )
{
   PHB_ITEM pCodeBlock = hb_param( 4, HB_IT_BLOCK );
   if( !pCodeBlock )
   {
      hb_retl( 0 );
      return;
   }

   HB_MAXINT nStart = hb_parnint( 1 );
   HB_MAXINT nEnd   = hb_parnint( 2 );
   HB_MAXINT nStep  = hb_parnintdef(3, 1);

   // basic parameter check for avoidance of possible infinite loop
   // or ineffectual/futile param values;
   // a more intensive parameter check might be necessary/wanted here!
   if( (nStep==0) || (nStep<0 && nStart<nEnd) || (nStep>0 && nStart>nEnd) )
   {
      hb_retl( 0 );
      return;
   }

   while( nStep<0 ? nStart >= nEnd : nStart <= nEnd )
   {
      hb_vmPushEvalSym();
      hb_vmPush( pCodeBlock );
      hb_vmPushNumInt( nStart );
      hb_vmSend( 1 );

      nStart += nStep;
   }
   hb_retl( 1 );
}

#pragma enddump
